/*
 * addr2if_inet6: Function to format a given address to a /proc/net/if_inet6 format
 *
 * Version:		$Id: addr2if_inet6.c,v 0.03 2001/02/27 $
 * 
 * Author:		Peter Bieringer <pb@bieringer.de>
 *
 * some hints taken from ifconfig.c (net-tools)
 * 
 * Credits to:
 *	net-tools authors 
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <netdb.h>
#include <linux/ipv6.h>
#include "ipv6calc.h"

/* function formats an given IPv6 address to /proc/net/if_inet6 format
 * examples:
 *  3ffe:400:100:f101::1 -> 3ffe04000100f1010000000000000001
 *
 * in : *addrstring = IPv6 address
 * out: *resultstring = result
 * ret: ==0: ok, !=0: error
 */
int addr2if_inet6(char *addrstring, char *resultstring) {
	int retval = 1, result;
	char tempstring[NI_MAXHOST], *addronlystring, *cp;
	int prefixlength = 0, flag_prefixlength = 0;
	unsigned int addr6p[8];

#ifdef DEBUG_addr2if_inet6
	fprintf(stderr, "addr2if_inet6: Got input %s\n", addrstring);
#endif
	
	/* first do an uncompressing */
	result = addr2uncompaddr(addrstring, tempstring);
	
#ifdef DEBUG_addr2if_inet6
	fprintf(stderr, "addr2if_inet6: Got temp result from uncompress: %s Result: %d\n", tempstring, result);
#endif

	if ( result != 0 ) {
		sprintf(resultstring, "%s", tempstring);
		retval = 1;
		return (retval);
	}

    /* save prefix length first, if available */
    addronlystring = strtok (tempstring, "/");
	cp = strtok (NULL, "/");
    if ( cp != NULL ) {
		prefixlength = atol(cp);
		if (prefixlength < 0 || prefixlength > 128 ) {
			sprintf(resultstring, "illegal prefix length");
			retval = 1;
			return (retval);
		}
		flag_prefixlength = 1;
	}

	/* scan address into array */
	result = sscanf(addronlystring, "%x:%x:%x:%x:%x:%x:%x:%x", &addr6p[0], &addr6p[1], &addr6p[2], &addr6p[3], &addr6p[4], &addr6p[5], &addr6p[6], &addr6p[7]);

#ifdef DEBUG_addr2if_inet6
	fprintf(stderr, "addr2ifinet6: Reading into array, got items: %d\n", result);
#endif

	if ( result != 8 ) {
		sprintf(resultstring, "error splitting address %s, got only %d items!", addronlystring, result);
		retval = 1;
		return (retval);
	}

	/* print out array */	
	if ( flag_prefixlength == 1 ) {
	    sprintf(resultstring, "%04x%04x%04x%04x%04x%04x%04x%04x %s %d", addr6p[0], addr6p[1], addr6p[2], addr6p[3], addr6p[4], addr6p[5], addr6p[6], addr6p[7], "ReservedForAddressTypeValue", prefixlength);
	} else {
	    sprintf(resultstring, "%04x%04x%04x%04x%04x%04x%04x%04x %s", addr6p[0], addr6p[1], addr6p[2], addr6p[3], addr6p[4], addr6p[5], addr6p[6], addr6p[7], "ReservedForAddressTypeValue");
	}

#ifdef DEBUG_addr2if_inet6
	fprintf(stderr, "addr2if_inet6: Print out: %s\n", resultstring);
#endif

	retval = 0;
	return (retval);
}

