/*
 * ipv6calc	This program print out different formats of an given IPv6 address
 *
 * Version:     $Id: ipv6calc.c,v 0.03 2001/02/27 $
 * 
 * Author:      Peter Bieringer <pb@bieringer.de>
 * 
 */

#include <stdio.h>
#include <string.h>
#include <netdb.h>
#include "ipv6calc.h"

/* display help */
void help() {
	fprintf(stderr, "%s  version: %s\n", PROGRAM_NAME, PROGRAM_VERSION);
	fprintf(stderr, "\nUsage:\n");
	fprintf(stderr, "\n %s --addr2ip6_int  ipv6addr[/prefixlength]\n", PROGRAM_NAME);
	fprintf(stderr, "  Converts given IPv6 address to a dot separated reverse nibble format\n");
	fprintf(stderr, "   e.g. 3ffe:400:100:f101::1 -> 1.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.1.0.1.f.0.0.1.0.0.0.4.0.e.f.f.3.ip6.int\n");
	fprintf(stderr, "   e.g. 3ffe:400:100:f101::1/64 -> 1.0.1.f.0.0.1.0.0.0.4.0.e.f.f.3.ip6.int\n");

	fprintf(stderr, "\n %s --addr2compaddr ipv6addr[/prefixlength]\n", PROGRAM_NAME);
	fprintf(stderr, "  Converts given IPv6 address to a compressed one\n");
	fprintf(stderr, "   e.g. 3ffe:0400:0100:f101:0000:0000:0000:0001 -> 3ffe:400:100:f101::1\n");
	
	fprintf(stderr, "\n %s --addr2uncompaddr ipv6addr[/prefixlength]\n", PROGRAM_NAME);
	fprintf(stderr, "  Converts given IPv6 address to an uncompressed one\n");
	fprintf(stderr, "   e.g. 3ffe:400:100:f101::1 -> 3ffe:400:100:f101:0:0:0:1\n");
	
	fprintf(stderr, "\n %s --addr2fulluncompaddr ipv6addr[/prefixlength]\n", PROGRAM_NAME);
	fprintf(stderr, "  Converts given IPv6 address to an uncompressed one\n");
	fprintf(stderr, "   e.g. 3ffe:400:100:f101::1 -> 3ffe:0400:0100:f101:0000:0000:0000:0001\n");
	
	fprintf(stderr, "\n %s --addr2if_inet6 ipv6addr[/prefixlength]\n", PROGRAM_NAME);
	fprintf(stderr, "  Converts given IPv6 address to the format shown in /proc/net/if_inet6\n");
	fprintf(stderr, "   e.g. 3ffe:400:100:f101::1 -> 3ffe04000100f1010000000000000001 Reserved\n");
	fprintf(stderr, "   e.g. 3ffe:400:100:f101::1/64 -> 3ffe04000100f1010000000000000001 ReservedForType 40\n");

	fprintf(stderr, "\n");

	return;
}

/**************************************************/
/* main */

int main(int argc,char *argv[]) {
	char resultstring[NI_MAXHOST];
	int retval = 1;


	if (argc <= 1) {
		fprintf(stderr, "missing selection!\n\n");
		help();
		exit(1);
	}

	/* do work depending on selection */
	if ( !strcmp(argv[1], "--help") || !strcmp(argv[1], "-h") || !strcmp(argv[1], "-?")) {
		help();
		exit(1);
	} else if ( !strcmp(argv[1], "--addr2ip6_int") ) {
		if (argc <= 2) {
			fprintf(stderr, "missing argument!\n\n");
			help();
			exit(1);
		}
		retval = addr2ip6_int(argv[2], resultstring);
		fprintf(stdout, "%s\n", resultstring);
		
	} else if ( !strcmp(argv[1], "--addr2compaddr")	) {
		if (argc <= 2) {
			fprintf(stderr, "missing argument!\n\n");
			help();
			exit(1);
		}
		retval = addr2compaddr(argv[2], resultstring);
		fprintf(stdout, "%s\n", resultstring);
		
	} else if ( !strcmp(argv[1], "--addr2uncompaddr")	) {
		if (argc <= 2) {
			fprintf(stderr, "missing argument!\n\n");
			help();
			exit(1);
		}
		retval = addr2uncompaddr(argv[2], resultstring);
		fprintf(stdout, "%s\n", resultstring);
		
	} else if ( !strcmp(argv[1], "--addr2fulluncompaddr")	) {
		if (argc <= 2) {
			fprintf(stderr, "missing argument!\n\n");
			help();
			exit(1);
		}
		retval = addr2fulluncompaddr(argv[2], resultstring);
		fprintf(stdout, "%s\n", resultstring);
		
	} else if ( !strcmp(argv[1], "--addr2if_inet6") ) {
		if (argc <= 2) {
			fprintf(stderr, "missing argument!\n\n");
			help();
			exit(1);
		}
		retval = addr2if_inet6(argv[2], resultstring);
		fprintf(stdout, "%s\n", resultstring);
		
	} else {
		fprintf(stderr, "unknown selection: %s\n\n", argv[1]);
		help();
		exit(1);
	}

	exit(retval);
}
