/*
 * librfc3041: Function libary for host identifier
 *              privacy extension defined in RFC 3041
 *
 * Version:		$Id: librfc3041.c,v 0.24 2001/04/29 $
 * 
 * Author:		Peter Bieringer <pb@bieringer.de>
 *
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <openssl/md5.h>
#include "ipv6calc.h"

/* function 
 *
 * in : *addrstring = IPv6 address
 * out: *resultstring = result
 * ret: ==0: ok, !=0: error
 */


int librfc3041_calc(ipv6calc_ipv6addr *identifier, ipv6calc_ipv6addr *token, ipv6calc_ipv6addr *newidentifier, ipv6calc_ipv6addr *newtoken) {
#define DEBUG_function_name "librfc3041/librfc3041_calc"
	int retval = 1, i;
	MD5_CTX md5hash;
	char tempstring[NI_MAXHOST],  tempstring2[NI_MAXHOST];
	unsigned char digest[MD5_DIGEST_LENGTH];

	if (ipv6calc_debug & DEBUG_librfc3041) {
		fprintf(stderr, "%s: Got identifier '%04x%04x%04x%04x' and token '%04x%04x%04x%04x'\n", DEBUG_function_name, identifier->addr6[4], identifier->addr6[5], identifier->addr6[6], identifier->addr6[7], token->addr6[4], token->addr6[5], token->addr6[6], token->addr6[7]); 
	};

	MD5_Init(&md5hash);

	sprintf(tempstring, "%04x%04x%04x%04x%04x%04x%04x%04x", identifier->addr6[4], identifier->addr6[5], identifier->addr6[6], identifier->addr6[7], token->addr6[4], token->addr6[5], token->addr6[6], token->addr6[7]);
	if (ipv6calc_debug & DEBUG_librfc3041) {
		fprintf(stderr, "%s: MD5 input '%s' (%d)\n", DEBUG_function_name, tempstring, strlen(tempstring));
	};
	MD5_Update(&md5hash, tempstring, strlen(tempstring) );

	MD5_Final(digest, &md5hash);

	sprintf(tempstring, "%s", "");
	for (i = 0; i < MD5_DIGEST_LENGTH; i++) {
		sprintf(tempstring2, "%s%02x", tempstring, digest[i]);
		strcpy(tempstring, tempstring2);
	};
	
	if (ipv6calc_debug & DEBUG_librfc3041) {
		fprintf(stderr, "%s: MD5 hash '%s'\n", DEBUG_function_name, tempstring);
	};
	
	newidentifier->addr6[4] = digest[0] << 8 | digest[1];
	newidentifier->addr6[5] = digest[2] << 8 | digest[3];
	newidentifier->addr6[6] = digest[4] << 8 | digest[5];
	newidentifier->addr6[7] = digest[6] << 8 | digest[7];
	
	newtoken->addr6[4] = digest[8]  << 8 | digest[9];
	newtoken->addr6[5] = digest[10] << 8 | digest[11];
	newtoken->addr6[6] = digest[12] << 8 | digest[13];
	newtoken->addr6[7] = digest[14] << 8 | digest[15];

   	retval = 0;	
	return (retval);
};

