/*
 *  ipv6types.c:
 *
 * Version:     $Id: ipv6types.c,v 0.24 2001/04/29 $
 *
 * Author:      Peter Bieringer <pb@bieringer.de>
 *
 * with credits to kernel and USAGI developer team
 * some code is taken from "kernel/net/ipv6/addrconf.c"
 */   

#include "ipv6calc.h"
#include "ipv6types.h"

unsigned int ipv6types_calc(ipv6calc_ipv6addr *ipv6addr) {
	unsigned int st;
	unsigned int type = 0;

	st = ipv6addr->addr6[0]; /* first 16 bits */

	/* Consider all addresses with the first three bits different of
	   000 and 111 as unicasts.
	 */
	if ((st & (0xE000)) != (0x0000) && (st & (0xE000)) != (0xE000)) {
		type |= IPV6_ADDR_UNICAST;
	};

	if ((st & (0xFF00)) == (0xFF00)) {
		type |= IPV6_ADDR_MULTICAST;

		switch((st & (0x00FF))) {
			case (0x0001):
				type |= IPV6_ADDR_LOOPBACK;
				break;

			case (0x0002):
				type |= IPV6_ADDR_LINKLOCAL;
				break;

			case (0x0005):
				type |= IPV6_ADDR_SITELOCAL;
				break;
		};
	};
	
	if ((st & (0xFFC0)) == (0xFE80)) {
		type |= (IPV6_ADDR_LINKLOCAL | IPV6_ADDR_UNICAST);
	};

	if ((st & (0xFFC0)) == (0xFEC0)) {
		type |= (IPV6_ADDR_SITELOCAL | IPV6_ADDR_UNICAST);
	};

	if (st == 0x3FFE) {
		/* 3ffe:... */
		type |= IPV6_NEW_ADDR_6BONE;
	};

	if (st == 0x2002) {
		/* 2002:... */
		type |= IPV6_NEW_ADDR_6TO4;
	};

/*
	if ((addr->s6_addr32[0] | addr->s6_addr32[1]) == 0) {
		if (addr->s6_addr32[2] == 0) {
			if (addr->in6_u.u6_addr32[3] == 0)
				return IPV6_ADDR_ANY;

			if (addr->s6_addr32[3] == (0x00000001))
				return (IPV6_ADDR_LOOPBACK | IPV6_ADDR_UNICAST);

			return (IPV6_ADDR_COMPATv4 | IPV6_ADDR_UNICAST);
		}

		if (addr->s6_addr32[2] == (0x0000ffff))
			return IPV6_ADDR_MAPPED;
	}
*/
	return (type);
/*	return IPV6_ADDR_RESERVED;*/
};

