/* 
 * ipv6types.h:
 *
 * Version:     $Id: ipv6types.h,v 0.24 2001/04/29 $
 * 
 * Author:      Peter Bieringer <pb@bieringer.de>
 * 
 * with credits to kernel and USAGI developer team
 * some information are taken from "kernel/include/net/ipv6.h"
 */ 

/*
 *	Addr type
 *	
 *	type	-	unicast | multicast | anycast
 *	scope	-	local	| site	    | global
 *	v4	-	compat
 *	v4mapped
 *	any
 *	loopback
 */

#ifndef _IPV6TYPES

#define _IPV6TYPES 1

#define IPV6_ADDR_ANY				0x0000U

#define IPV6_ADDR_UNICAST      		0x0001U	
#define IPV6_ADDR_MULTICAST    		0x0002U	
#define IPV6_ADDR_ANYCAST			0x0004U

#define IPV6_ADDR_LOOPBACK			0x0010U
#define IPV6_ADDR_LINKLOCAL			0x0020U
#define IPV6_ADDR_SITELOCAL			0x0040U

#define IPV6_ADDR_COMPATv4			0x0080U

#define IPV6_ADDR_SCOPE_MASK		0x00f0U

#define IPV6_ADDR_MAPPED			0x1000U
#define IPV6_ADDR_RESERVED			0x2000U	/* reserved address space */

#define IPV6_NEW_ADDR_6TO4			0x10000U
#define IPV6_NEW_ADDR_6BONE			0x20000U
#define IPV6_NEW_ADDR_AGU			0x40000U

/* text representations */
#define TXT_IPV6_ADDR_ANY			"unknown"
#define TXT_IPV6_ADDR_UNICAST		"unicast"
#define TXT_IPV6_ADDR_MULTICAST		"multicast"
#define TXT_IPV6_ADDR_ANYCAST		"anycast"
#define TXT_IPV6_ADDR_LOOPBACK		"loopback"
#define TXT_IPV6_ADDR_LINKLOCAL		"link-local"
#define TXT_IPV6_ADDR_SITELOCAL		"site-local"
#define TXT_IPV6_ADDR_COMPATv4		"compat-v4"
#define TXT_IPV6_ADDR_MAPPED		"mapped"
#define TXT_IPV6_ADDR_RESERVED		"reserved"
#define TXT_IPV6_NEW_ADDR_6TO4		"6to4"
#define TXT_IPV6_NEW_ADDR_6BONE		"6bone"
#define TXT_IPV6_NEW_ADDR_AGU		"aggregatable-global-unicast"

#define IPV6INFO_NUM 12

/* array of numerical types */
typedef unsigned int test;
test ipv6types_num[IPV6INFO_NUM] = {
    IPV6_ADDR_UNICAST,
	IPV6_ADDR_MULTICAST,
	IPV6_ADDR_ANYCAST,
	IPV6_ADDR_LOOPBACK,
	IPV6_ADDR_LINKLOCAL,
	IPV6_ADDR_SITELOCAL,
	IPV6_ADDR_COMPATv4,
	IPV6_ADDR_MAPPED,
	IPV6_ADDR_RESERVED,
	IPV6_NEW_ADDR_6TO4,
	IPV6_NEW_ADDR_6BONE, 
	IPV6_NEW_ADDR_AGU
};

char *ipv6types_string[IPV6INFO_NUM] = {
    TXT_IPV6_ADDR_UNICAST,
	TXT_IPV6_ADDR_MULTICAST,
	TXT_IPV6_ADDR_ANYCAST,
	TXT_IPV6_ADDR_LOOPBACK,
	TXT_IPV6_ADDR_LINKLOCAL,
	TXT_IPV6_ADDR_SITELOCAL,
	TXT_IPV6_ADDR_COMPATv4,
	TXT_IPV6_ADDR_MAPPED,
	TXT_IPV6_ADDR_RESERVED,
	TXT_IPV6_NEW_ADDR_6TO4,
	TXT_IPV6_NEW_ADDR_6BONE,
	TXT_IPV6_NEW_ADDR_AGU
};


#endif
